/* 

                          Firewall Builder

                 Copyright (C) 2009 NetCitadel, LLC

  Author:  Vadim Kurland     vadim@fwbuilder.org

  $Id: FailoverClusterGroup.h 534 2010-02-07 23:25:06Z vadim $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#ifndef __FAILOVERCLUSTERGROUP_HH_
#define __FAILOVERCLUSTERGROUP_HH_

#include <fwbuilder/ClusterGroup.h>

namespace libfwbuilder
{
    class FailoverClusterGroup : public ClusterGroup
    {
    public:
        FailoverClusterGroup();
        FailoverClusterGroup(const FWObjectDatabase *root, bool prepopulate);
        virtual ~FailoverClusterGroup() {};

        DECLARE_FWOBJECT_SUBTYPE(FailoverClusterGroup);

        DECLARE_DISPATCH_METHODS(FailoverClusterGroup);
        
        virtual void fromXML(xmlNodePtr parent) throw(FWException);
        virtual xmlNodePtr toXML(xmlNodePtr parent) throw(FWException);
    };
}

#endif /* __CLUSTERGROUP_HH_ */

