/* 

                          Firewall Builder

                 Copyright (C) 2000 NetCitadel, LLC

  Author:  Vadim Kurland     vadim@fwbuilder.org

  $Id: FWIntervalReference.h 534 2010-02-07 23:25:06Z vadim $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#ifndef  __FWINTERVALREF_HH_FLAG__
#define  __FWINTERVALREF_HH_FLAG__

#include <fwbuilder/FWReference.h>
#include <fwbuilder/Interval.h>
#include <fwbuilder/IntervalGroup.h>

namespace libfwbuilder
{

/**
 * This class represents object reference.
 */
class FWIntervalReference : public FWReference
{
    public:

    DECLARE_FWOBJECT_SUBTYPE(FWIntervalReference);

    DECLARE_DISPATCH_METHODS(FWIntervalReference);
    
    FWIntervalReference();
    FWIntervalReference(const FWObjectDatabase *root,bool prepopulate);

    void setPointer(Interval *o);
    void setPointer(IntervalGroup *o);

};

}

#endif // _FWOBJECT_HH
